@@ -34,6 +34,9 @@
 #define BUF_MAX_INCR 1000000000
 #define BUF_MAX_SIZE 1000000000
 
+/* avoid excessively large numbers, > ~8192 bit */
+#define BUF_MAX_MPINT (8200 / 8)
+
 /* Create (malloc) a new buffer of size */
 buffer* buf_new(unsigned int size) {
 
@@ -73,7 +76,8 @@ void buf_burn(buffer* buf) {
 
 }
 
-/* resize a buffer, pos and len will be repositioned if required */
+/* resize a buffer, pos and len will be repositioned if required when
+ * downsizing */
 void buf_resize(buffer *buf, unsigned int newsize) {
 
 	if (newsize > BUF_MAX_SIZE) {
@@ -148,6 +152,8 @@ void buf_incrpos(buffer* buf,  int incr) {
 /* Get a byte from the buffer and increment the pos */
 unsigned char buf_getbyte(buffer* buf) {
 
+	/* This check is really just "==", but the >= allows us to check for the
+	 * assert()able case of pos > len, which should _never_ happen */
 	if (buf->pos >= buf->len) {
 		dropbear_exit("bad buf_getbyte");
 	}
@@ -157,7 +163,8 @@ unsigned char buf_getbyte(buffer* buf) {
 /* put a byte, incrementing the length if required */
 void buf_putbyte(buffer* buf, unsigned char val) {
 
-	if (buf->pos >= buf->len) {
+	/* assert(buf->pos <= buf->len) */
+	if (buf->pos == buf->len) {
 		buf_incrlen(buf, 1);
 	}
 	buf->data[buf->pos] = val;
@@ -297,6 +304,10 @@ int buf_getmpint(buffer* buf, mp_int* mp) {
 		return DROPBEAR_SUCCESS;
 	}
 
+	if (len > BUF_MAX_MPINT) {
+		return DROPBEAR_FAILURE;
+	}
+
 	/* check for negative */
 	if (*buf_getptr(buf, 1) & (1 << (CHAR_BIT-1))) {
 		return DROPBEAR_FAILURE;
