@@ -36,6 +36,7 @@
 #include "x11fwd.h"
 #include "agentfwd.h"
 #include "localtcpfwd.h"
+#include "remotetcpfwd.h"
 
 static void send_msg_channel_open_failure(unsigned int remotechan, int reason,
 		const unsigned char *text, const unsigned char *lang);
@@ -99,7 +100,7 @@ struct Channel* newchannel(unsigned int remotechan, unsigned char type,
 	struct Channel * newchan;
 	unsigned int i, j;
 
-	TRACE(("enter newchannel"));
+	TRACE(("enter newchannel - remotechan %d, type %d", remotechan, type));
 	
 	/* first see if we can use existing channels */
 	for (i = 0; i < ses.chansize; i++) {
@@ -110,7 +111,7 @@ struct Channel* newchannel(unsigned int remotechan, unsigned char type,
 
 	/* otherwise extend the list */
 	if (i == ses.chansize) {
-		if (ses.chansize > MAX_CHANNELS) {
+		if (ses.chansize >= MAX_CHANNELS) {
 			TRACE(("leave newchannel: max chans reached"));
 			return NULL;
 		}
@@ -192,22 +193,22 @@ void channelio(fd_set *readfd, fd_set *writefd) {
 		 * see if it has errors */
 		if (channel->infd >= 0 && channel->infd != channel->outfd
 				&& FD_ISSET(channel->infd, readfd)) {
-			int ret;
-			ret = write(channel->infd, NULL, 0);
-			if (ret < 0 && errno != EINTR && errno != EAGAIN) {
-				closeinfd(channel);
+			if (channel->initconn) {
+				/* Handling for "in progress" connections. */
+				checkinitdone(channel);
+				continue; /* Important not to use the channel after
+							 checkinitdone(), as it may be NULL */
+			} else {
+				if ( (write(channel->infd, NULL, 0) < 0)
+							&& errno != EINTR && errno != EAGAIN) {
+					closeinfd(channel);
+				}
 			}
 		}
 
 		/* write to program/pipe stdin */
 		if (channel->infd >= 0 && FD_ISSET(channel->infd, writefd)) {
-			if (channel->initconn) {
-				checkinitdone(channel);
-				continue; /* Important not to use the channel after
-							 checkinitdone(), as it may be NULL */
-			} else {
-				writechannel(channel);
-			}
+			writechannel(channel);
 		}
 	
 		/* handle any listening sockets */
@@ -222,6 +223,9 @@ void channelio(fd_set *readfd, fd_set *writefd) {
 			if (chansess->agentfd >= 0 && FD_ISSET(chansess->agentfd,readfd)) {
 				agentaccept(chansess);
 			}
+#endif
+#ifndef DISABLE_REMOTETCPFWD
+			handleremotetcp(readfd);
 #endif
 		}
 
@@ -235,6 +239,7 @@ void channelio(fd_set *readfd, fd_set *writefd) {
 /* do all the EOF/close type stuff checking for a channel */
 static void checkclose(struct Channel *channel) {
 
+	TRACE(("enter checkclose"));
 	if (!channel->sentclosed) {
 
 		/* check for exited */
@@ -257,9 +262,24 @@ static void checkclose(struct Channel *channel) {
 		}
 	}
 
-	if (channel->sentclosed && channel->recvclosed) {
+	/* When either party wishes to terminate the channel, it sends
+	 * SSH_MSG_CHANNEL_CLOSE.  Upon receiving this message, a party MUST
+	 * send back a SSH_MSG_CHANNEL_CLOSE unless it has already sent this
+	 * message for the channel.  The channel is considered closed for a
+	 * party when it has both sent and received SSH_MSG_CHANNEL_CLOSE, and
+	 * the party may then reuse the channel number.  A party MAY send
+	 * SSH_MSG_CHANNEL_CLOSE without having sent or received
+	 * SSH_MSG_CHANNEL_EOF. 
+	 * (from draft-ietf-secsh-connect)
+	 */
+	if (channel->recvclosed) {
+		if (! channel->sentclosed) {
+			TRACE(("Sending MSG_CHANNEL_CLOSE in response to same."));
+			send_msg_channel_close(channel);
+		}
 		removechannel(channel);
 	}
+	TRACE(("leave checkclose"));
 }
 
 
@@ -276,10 +296,14 @@ static void checkinitdone(struct Channel *channel) {
 
 	if (getsockopt(channel->infd, SOL_SOCKET, SO_ERROR, &val, &vallen)
 			|| val != 0) {
+		send_msg_channel_open_failure(channel->remotechan,
+				SSH_OPEN_CONNECT_FAILED, "", "");
 		close(channel->infd);
 		deletechannel(channel);
 		TRACE(("leave checkinitdone: fail"));
 	} else {
+		send_msg_channel_open_confirmation(channel, channel->recvwindow,
+			channel->recvmaxpacket);
 		channel->outfd = channel->infd;
 		channel->initconn = 0;
 		TRACE(("leave checkinitdone: success"));
@@ -326,7 +350,7 @@ static void send_msg_channel_close(struct Channel *channel) {
 /* call this when trans/eof channels are closed */
 static void send_msg_channel_eof(struct Channel *channel) {
 
-	TRACE(("enter send_msg_channel_eof"));
+	TRACE(("enter send_msg_channel_eof - remotechan %d", channel->remotechan));
 	CHECKCLEARTOWRITE();
 
 	buf_putbyte(ses.writepayload, SSH_MSG_CHANNEL_EOF);
@@ -437,6 +461,9 @@ void setchannelfds(fd_set *readfd, fd_set *writefd) {
 			if (chansess->agentfd >= 0) {
 				FD_SET(chansess->agentfd, readfd);
 			}
+#endif
+#ifndef DISABLE_REMOTETCPFWD
+			setremotetcpfds(readfd);
 #endif
 		}
 	} /* foreach channel */
@@ -760,6 +787,10 @@ void recv_msg_channel_open() {
 		typeval = CHANNEL_ID_SESSION;
 #ifndef DISABLE_LOCALTCPFWD
 	} else if (strcmp(type, "direct-tcpip") == 0) {
+		if (ses.opts->nolocaltcp) {
+			errtype = SSH_OPEN_ADMINISTRATIVELY_PROHIBITED;
+			goto failure;
+		}
 		typeval = CHANNEL_ID_TCPDIRECT;
 #endif
 	} else {
@@ -780,9 +811,16 @@ void recv_msg_channel_open() {
 #ifndef DISABLE_LOCALTCPFWD
 	} else if (typeval == CHANNEL_ID_TCPDIRECT) {
 		if (newtcpdirect(channel) == DROPBEAR_FAILURE) {
+			errtype = SSH_OPEN_CONNECT_FAILED;
 			deletechannel(channel);
 			goto failure;
 		}
+		/* This is a special case for localtcpfwd. Even if we succeeded, we
+		 * don't want to send the confirmation yet, since the connection may
+		 * just be "in progress" (so we might want to send a failure message
+		 * later).  We just drop out here to 'cleanup:', and send the real
+		 * confirmation/failure in checkinitdone() */
+		goto cleanup;
 #endif
 	}
 
@@ -831,7 +869,7 @@ void send_msg_channel_success(struct Channel *channel) {
 static void send_msg_channel_open_failure(unsigned int remotechan, 
 		int reason, const unsigned char *text, const unsigned char *lang) {
 
-	TRACE(("enter send_msg_channel_open_failure"));
+	TRACE(("enter send_msg_channel_open_failure - remotechan %d", remotechan));
 	CHECKCLEARTOWRITE();
 	
 	buf_putbyte(ses.writepayload, SSH_MSG_CHANNEL_OPEN_FAILURE);
@@ -869,17 +907,21 @@ static void send_msg_channel_open_confirmation(struct Channel* channel,
  * options, with the calling function calling encrypt_packet() after
  * completion. It is mandatory for the caller to encrypt_packet() if
  * DROPBEAR_SUCCESS is returned */
-int send_msg_channel_open_init(int fd, const char * typestring) {
+int send_msg_channel_open_init(int fd, unsigned char type,
+		const char * typestring) {
 
 	struct Channel* chan;
 
-	chan = newchannel(0, CHANNEL_ID_AGENT, 0, 0);
+	TRACE(("enter send_msg_channel_open_init()"));
+	chan = newchannel(0, type, 0, 0);
 	if (!chan) {
+		TRACE(("leave send_msg_channel_open_init() - FAILED in newchannel()"));
 		return DROPBEAR_FAILURE;
 	}
 
 	/* set fd non-blocking */
 	if (fcntl(fd, F_SETFL, O_NONBLOCK) < 0) {
+		TRACE(("leave send_msg_channel_open_init() - FAILED in fcntl()"));
 		return DROPBEAR_FAILURE;
 	}
 
@@ -895,6 +937,7 @@ int send_msg_channel_open_init(int fd, const char * typestring) {
 	buf_putint(ses.writepayload, RECV_MAXWINDOW);
 	buf_putint(ses.writepayload, RECV_MAXPACKET);
 
+	TRACE(("leave send_msg_channel_open_init()"));
 	return DROPBEAR_SUCCESS;
 }
 
