@@ -15,26 +15,28 @@
 #include <tommath.h>
 
 /* returns size of ASCII reprensentation */
-int
-mp_radix_size (mp_int * a, int radix)
+int mp_radix_size (mp_int * a, int radix, int *size)
 {
   int     res, digs;
   mp_int  t;
   mp_digit d;
 
+  *size = 0;
+
   /* special case for binary */
   if (radix == 2) {
-    return mp_count_bits (a) + (a->sign == MP_NEG ? 1 : 0) + 1;
+    *size = mp_count_bits (a) + (a->sign == MP_NEG ? 1 : 0) + 1;
+    return MP_OKAY;
   }
 
   /* make sure the radix is in range */
   if (radix < 2 || radix > 64) {
-    return 0;
+    return MP_VAL;
   }
 
   /* init a copy of the input */
   if ((res = mp_init_copy (&t, a)) != MP_OKAY) {
-    return 0;
+    return res;
   }
 
   /* digs is the digit count */
@@ -57,6 +59,7 @@ mp_radix_size (mp_int * a, int radix)
   mp_clear (&t);
 
   /* return digs + 1, the 1 is for the NULL byte that would be required. */
-  return digs + 1;
+  *size = digs + 1;
+  return MP_OKAY;
 }
 
