@@ -15,8 +15,7 @@
 #include <tommath.h>
 
 /* set to a digit */
-void
-mp_set (mp_int * a, mp_digit b)
+void mp_set (mp_int * a, mp_digit b)
 {
   mp_zero (a);
   a->dp[0] = b & MP_MASK;
