@@ -23,68 +23,127 @@
  * SOFTWARE. */
 
 #include "includes.h"
-#include "util.h"
+#include "dbutil.h"
 #include "session.h"
 #include "buffer.h"
 #include "signkey.h"
 #include "runopts.h"
 
-static void listensocket(int *sock, uint16_t port);
+static int listensockets(int *sock, runopts * opts, int *maxfd);
 static void sigchld_handler(int dummy);
 static void sigsegv_handler(int);
 static void sigintterm_handler(int fish);
+static void main_inetd(runopts * opts);
+static void main_noinetd(runopts * opts);
+static void commonsetup();
 
 static int childpipes[MAX_UNAUTH_CLIENTS];
 
-int main(int argc, char ** argv) {
+#if defined(DBMULTI_DROPBEAR) || !defined(DROPBEAR_MULTI)
+#if defined(DBMULTI_DROPBEAR) && defined(DROPBEAR_MULTI)
+int dropbear_main(int argc, char ** argv)
+#else
+int main(int argc, char ** argv)
+#endif
+{
 	
+	runopts * opts;
+
+	/* get commandline options */
+	opts = getrunopts(argc, argv);
+
+#ifdef INETD_MODE
+	/* service program mode */
+	if (opts->inetdmode) {
+		main_inetd(opts);
+		/* notreached */
+	}
+#endif
+
+#ifdef NON_INETD_MODE
+		main_noinetd(opts);
+#endif
+
+	dropbear_exit("Compiled without normal mode, can't run without -i\n");
+	return 1;
+}
+
+#ifdef INETD_MODE
+static void main_inetd(runopts * opts) {
+
+	struct sockaddr remoteaddr;
+	int remoteaddrlen;
+	char * addrstring = NULL;
+
+	/* Set up handlers, syslog */
+	commonsetup();
+
+	remoteaddrlen = sizeof(remoteaddr);
+	if (getpeername(0, &remoteaddr, &remoteaddrlen) < 0) {
+		dropbear_exit("Unable to getpeername: %s", strerror(errno));
+	}
+
+	/* In case our inetd was lax in logging source addresses */
+	addrstring = getaddrstring(&remoteaddr);
+	dropbear_log(LOG_INFO, "Child connection from %s", addrstring);
+	m_free(addrstring);
+
+	/* Don't check the return value - it may just fail since inetd has
+	 * already done setsid() after forking (xinetd on Darwin appears to do
+	 * this */
+	setsid();
+
+	/* Start service program 
+	 * -1 is a dummy childpipe, just something we can close() without 
+	 * mattering. */
+	child_session(0, opts, -1, &remoteaddr);
+
+	/* notreached */
+}
+#endif /* NON_INETD_MODE */
+
+
+#ifdef NON_INETD_MODE
+static void main_noinetd(runopts * opts) {
+
 	fd_set fds;
 	struct timeval seltimeout;
-	int i, j, val;
-	int maxsock;
+	unsigned int i, j;
+	int val;
+	int maxsock = -1;
 	struct sockaddr remoteaddr;
 	int remoteaddrlen;
 	int listensocks[MAX_LISTEN_ADDR];
 	unsigned int listensockcount = 0;
-	runopts * opts;
-	FILE * pidfile;
+	FILE * pidfile = NULL;
+	char * addrstring = NULL;
 
 	int childsock;
 	pid_t childpid;
 	int childpipe[2];
 
-	struct sigaction sa_chld;
-
-	/* get commandline options */
-	opts = getrunopts(argc, argv);
-
-
-	/* fork to background, returning (interactive) users to a term */
+	/* fork */
 	if (opts->forkbg) {
-
-		switch (fork()) {
-			case -1:
-				dropbear_exit("Failed to create background process");
-			case 0:
-				break;
-			default:
-				exit(0);
+		int closefds = 0;
+#ifndef DEBUG_TRACE
+		if (!usingsyslog) {
+			closefds = 1;
 		}
-		if (setpgid(0,0) < 0) {
-			dropbear_exit("Failed to set process group");
+#endif
+		if (daemon(0, closefds) < 0) {
+			dropbear_exit("Failed to daemonize: %s", strerror(errno));
 		}
-		startsyslog();
-		fprintf(stderr,"Dropbear: Running in background.\n");
-	} else {
-		fprintf(stderr,"Dropbear: Not forking\n");
 	}
-	
-	/* setup the sockets - we're allowing for multiple listening sockets
-	 * so we can do ip6 etc in future */
-	listensocket(&listensocks[0], opts->port);
-	listensockcount = 1;
-	maxsock = listensocks[listensockcount-1];
 
+	/* Set up syslog and handlers */
+	commonsetup();
+
+	/* should be done after syslog is working */
+	if (opts->forkbg) {
+		dropbear_log(LOG_INFO, "Running in background");
+	} else {
+		dropbear_log(LOG_INFO, "Not forking");
+	}
 
 	/* create a PID file so that we can be killed easily */
 	pidfile = fopen(DROPBEAR_PIDFILE, "w");
@@ -93,33 +152,21 @@ int main(int argc, char ** argv) {
 		fclose(pidfile);
 	}
 
-	/* set up cleanup handler */
-	if (signal(SIGINT, sigintterm_handler) == SIG_ERR || 
-		signal(SIGTERM, sigintterm_handler) == SIG_ERR) {
-		dropbear_exit("signal() error");
-	}
-
-	/* catch and reap zombie children */
-	sa_chld.sa_handler = sigchld_handler;
-	sa_chld.sa_flags = SA_NOCLDSTOP;
-	if (sigaction(SIGCHLD, &sa_chld, NULL) < 0) {
-		dropbear_exit("signal() error");
-	}
-	if (signal(SIGSEGV, sigsegv_handler) == SIG_ERR) {
-		dropbear_exit("signal() error");
-	}
-
 	/* sockets to identify pre-authenticated clients */
 	for (i = 0; i < MAX_UNAUTH_CLIENTS; i++) {
 		childpipes[i] = -1;
 	}
+	
+	/* Set up the listening sockets */
+	/* XXX XXX ports */
+	listensockcount = listensockets(listensocks, opts, &maxsock);
 
 	/* incoming connection select loop */
 	for(;;) {
 
 		FD_ZERO(&fds);
 		
-		seltimeout.tv_sec = 60; /* TODO find good value */
+		seltimeout.tv_sec = 60;
 		seltimeout.tv_usec = 0;
 		
 		/* listening sockets */
@@ -129,9 +176,10 @@ int main(int argc, char ** argv) {
 
 		/* pre-authentication clients */
 		for (i = 0; i < MAX_UNAUTH_CLIENTS; i++) {
-			if (childpipes[i] >= 0) 
-			FD_SET(childpipes[i], &fds);
-			maxsock = MAX(maxsock, childpipes[i]);
+			if (childpipes[i] >= 0) {
+				FD_SET(childpipes[i], &fds);
+				maxsock = MAX(maxsock, childpipes[i]);
+			}
 		}
 
 		val = select(maxsock+1, &fds, NULL, NULL, &seltimeout);
@@ -145,6 +193,7 @@ int main(int argc, char ** argv) {
 			/* timeout reached */
 			continue;
 		}
+
 		if (val < 0) {
 			if (errno == EINTR) {
 				continue;
@@ -152,7 +201,8 @@ int main(int argc, char ** argv) {
 			dropbear_exit("Listening socket error");
 		}
 
-		/* close fds which have been authed or closed */
+		/* close fds which have been authed or closed - auth.c handles
+		 * closing the auth sockets on success */
 		for (i = 0; i < MAX_UNAUTH_CLIENTS; i++) {
 			if (childpipes[i] >= 0 && FD_ISSET(childpipes[i], &fds)) {
 				close(childpipes[i]);
@@ -165,10 +215,9 @@ int main(int argc, char ** argv) {
 			if (!FD_ISSET(listensocks[i], &fds)) 
 				continue;
 
-			/* child connection */
-			remoteaddrlen = sizeof(struct sockaddr_in);
-			childsock = accept(listensocks[i], 
-					&remoteaddr, &remoteaddrlen);
+			/* child connection XXX - ip6 stuff here */
+			remoteaddrlen = sizeof(remoteaddr);
+			childsock = accept(listensocks[i], &remoteaddr, &remoteaddrlen);
 
 			if (childsock < 0) {
 				/* accept failed */
@@ -181,6 +230,7 @@ int main(int argc, char ** argv) {
 					break;
 				}
 			}
+
 			if (j == MAX_UNAUTH_CLIENTS) {
 				/* no free connections */
 				/* TODO - possibly log, though this would be an easy way
@@ -202,8 +252,13 @@ int main(int argc, char ** argv) {
 				extern void _start(void), etext(void);
 				monstartup((u_long)&_start, (u_long)&etext);
 #endif /* DEBUG_FORKGPROF */
-				if (setpgid(0,0) < 0) {
-					dropbear_exit("Error creating child");
+
+				addrstring = getaddrstring(&remoteaddr);
+				dropbear_log(LOG_INFO, "Child connection from %s", addrstring);
+				m_free(addrstring);
+
+				if (setsid() < 0) {
+					dropbear_exit("setsid");
 				}
 
 				/* make sure we close sockets */
@@ -232,8 +287,98 @@ int main(int argc, char ** argv) {
 	} /* for(;;) loop */
 
 	/* don't reach here */
-	return -1;
 }
+#endif /* NON_INETD_MODE */
+
+/* Things used by inetd and non-inetd modes */
+static void commonsetup() {
+
+	struct sigaction sa_chld;
+#ifndef DISABLE_SYSLOG
+	if (usingsyslog) {
+		startsyslog();
+	}
+#endif
+
+	/* set up cleanup handler */
+	if (signal(SIGINT, sigintterm_handler) == SIG_ERR || 
+#ifndef DEBUG_VALGRIND
+		signal(SIGTERM, sigintterm_handler) == SIG_ERR ||
+#endif
+		signal(SIGPIPE, SIG_IGN) == SIG_ERR) {
+		dropbear_exit("signal() error");
+	}
+
+	/* catch and reap zombie children */
+	sa_chld.sa_handler = sigchld_handler;
+	sa_chld.sa_flags = SA_NOCLDSTOP;
+	if (sigaction(SIGCHLD, &sa_chld, NULL) < 0) {
+		dropbear_exit("signal() error");
+	}
+	if (signal(SIGSEGV, sigsegv_handler) == SIG_ERR) {
+		dropbear_exit("signal() error");
+	}
+}
+
+/* Set up listening sockets for all the requested ports */
+static int listensockets(int *sock, runopts * opts, int *maxfd) {
+	
+	int listensock; /* listening fd */
+	struct sockaddr_in listen_addr;
+	struct linger linger;
+	unsigned int i;
+	int val;
+
+	for (i = 0; i < opts->portcount; i++) {
+
+		/* iterate through all the sockets to listen on */
+		listensock = socket(PF_INET, SOCK_STREAM, 0);
+		if (listensock < 0) {
+			dropbear_exit("Failed to create socket");
+		}
+
+		val = 1;
+		/* set to reuse, quick timeout */
+		setsockopt(listensock, SOL_SOCKET, SO_REUSEADDR,
+				(void*) &val, sizeof(val));
+		linger.l_onoff = 1;
+		linger.l_linger = 5;
+		setsockopt(listensock, SOL_SOCKET, SO_LINGER,
+				(void*)&linger, sizeof(linger));
+
+		/* disable nagle */
+		setsockopt(listensock, IPPROTO_TCP, TCP_NODELAY,
+				(void*)&val, sizeof(val));
+
+		memset((void*)&listen_addr, 0x0, sizeof(listen_addr));
+		listen_addr.sin_family = AF_INET;
+		listen_addr.sin_port = htons(opts->ports[i]);
+		listen_addr.sin_addr.s_addr = htonl(INADDR_ANY);
+		memset(&(listen_addr.sin_zero), '\0', 8);
+
+		if (bind(listensock, (struct sockaddr *)&listen_addr,
+					sizeof(listen_addr)) < 0) {
+			dropbear_exit("Bind failed port %d", opts->ports[i]);
+		}
+
+		/* listen */
+		if (listen(listensock, 20) < 0) { /* TODO set listen count */
+			dropbear_exit("Listen failed");
+		}
+
+		/* nonblock */
+		if (fcntl(listensock, F_SETFL, O_NONBLOCK) < 0) {
+			dropbear_exit("Failed to set non-blocking");
+		}
+
+		sock[i] = listensock;
+		*maxfd = MAX(listensock, *maxfd);
+	}
+
+	return opts->portcount;
+}
+
+#endif  /* NON_INETD_MODE */
 
 /* catch + reap zombie children */
 static void sigchld_handler(int fish) {
@@ -255,57 +400,9 @@ static void sigsegv_handler(int fish) {
 	exit(EXIT_FAILURE);
 }
 
-/* catch ctrl-c or kill */
+/* catch ctrl-c or sigterm */
 static void sigintterm_handler(int fish) {
 
 	exitflag = 1;
 }
 
-static void listensocket(int *sock, uint16_t port) {
-	
-	int listensock; /* listening fd */
-	struct sockaddr_in listen_addr;
-	struct linger linger;
-	int val;
-
-	/* open a socket */
-	listensock = socket(PF_INET, SOCK_STREAM, 0);
-	if (listensock < 0) {
-		dropbear_exit("Failed to create socket");
-	}
-
-	/* set to reuse, quick timeout */
-	setsockopt(listensock, SOL_SOCKET, SO_REUSEADDR,
-			(void*) &val, sizeof(val));
-	linger.l_onoff = 1;
-	linger.l_linger = 5;
-	setsockopt(listensock, SOL_SOCKET, SO_LINGER,
-			(void*)&linger, sizeof(linger));
-
-	val = 1;
-	/* should really use getprotbyname, but we'd need to change "tcp" anyway */
-	setsockopt(listensock, 6, TCP_NODELAY, (void*)&val, sizeof(val));
-
-	/* bind all ip4 addresses */
-	listen_addr.sin_family = AF_INET;
-	listen_addr.sin_port = htons(port);
-	listen_addr.sin_addr.s_addr = htonl(INADDR_ANY);
-	memset(&(listen_addr.sin_zero), '\0', 8); /* XXX neccesary? */
-
-	if (bind(listensock, (struct sockaddr *)&listen_addr, 
-				sizeof(struct sockaddr)) < 0) {
-		dropbear_exit("Bind failed");
-	}
-
-	/* listen */
-	if (listen(listensock, 20) < 0) { /* TODO set listen count */
-		dropbear_exit("Listen failed");
-	}
-
-	/* nonblock */
-	if (fcntl(listensock, F_SETFL, O_NONBLOCK) < 0) {
-		dropbear_exit("Failed to set non-blocking");
-	}
-
-	*sock = listensock;
-}
