@@ -42,11 +42,30 @@
 #define RSA_PRIV_FILENAME "/etc/dropbear/dropbear_rsa_host_key"
 #endif
 
-/* Setting this disables the fast exptmod bignum code. It saves ~5kB, but is
- * perhaps 20% slower for pubkey operations (it is probably worth experimenting
- * if you want to use this) */
+/* Set NON_INETD_MODE if you require daemon functionality (ie Dropbear listens
+ * on chosen ports and keeps accepting connections. This is the default.
+ *
+ * Set INETD_MODE if you want to be able to run Dropbear with inetd (or
+ * similar), where it will use stdin/stdout for connections, and each process
+ * lasts for a single connection. Dropbear should be invoked with the -i flag
+ * for inetd, and can only accept IPv4 connections.
+ *
+ * Both of these flags can be defined at once, don't compile without at least
+ * one of them. */
+#define NON_INETD_MODE
+#define INETD_MODE
+
+/* Setting this disables the faster version of the modular exponentiation
+ * bignum code. It saves ~5kB, but is perhaps 20% slower for public-key and key
+ * exchange operations.  It is probably worth experimenting to decide if it's
+ * worthwhile on your platform. */
 /*#define NO_FAST_EXPTMOD*/
 
+/* Set this if you want to use the DROPBEAR_SMALL_CODE option. This can save
+ * several kB in binary size, however will make the symmetrical ciphers (AES,
+ * DES etc) slower (perhaps by 50%). Recommended for most small systems. */
+#define DROPBEAR_SMALL_CODE
+
 /* Enable X11 Forwarding */
 #define ENABLE_X11FWD
 
@@ -77,7 +96,7 @@
  * These are also used for key fingerprints in logs (when pubkey auth is used),
  * MD5 fingerprints are printed by default, however SHA1 fingerprints will be
  * generated otherwise. This isn't exactly optimal, although sha1 fingerprints
- * are not too hard go create from pubkeys if required. */
+ * are not too hard to create from pubkeys if required. */
 #define DROPBEAR_SHA1_HMAC
 #define DROPBEAR_MD5_HMAC
 
@@ -179,7 +198,7 @@
  *******************************************************************/
 
 #ifndef DROPBEAR_VERSION
-#define DROPBEAR_VERSION "0.40"
+#define DROPBEAR_VERSION "0.42"
 #endif
 
 #define LOCAL_IDENT "SSH-2.0-dropbear_" DROPBEAR_VERSION
