@@ -25,7 +25,7 @@
 #include "includes.h"
 #include "packet.h"
 #include "session.h"
-#include "util.h"
+#include "dbutil.h"
 #include "ssh.h"
 #include "algo.h"
 #include "buffer.h"
@@ -75,7 +75,7 @@ void write_packet() {
 	} 
 
 	if (written == 0) {
-		dropbear_close("remote host closed connection");
+		session_remoteclosed();
 	}
 
 	if (written == len) {
@@ -122,10 +122,9 @@ void read_packet() {
 	assert(ses.readbuf != NULL);
 	maxlen = ses.readbuf->len - ses.readbuf->pos;
 	len = read(ses.sock, buf_getptr(ses.readbuf, maxlen), maxlen);
-	buf_incrpos(ses.readbuf, len);
 
 	if (len == 0) {
-		dropbear_close("remote host closed connection");
+		session_remoteclosed();
 	}
 
 	if (len < 0) {
@@ -133,10 +132,12 @@ void read_packet() {
 			TRACE(("leave read_packet: EINTR or EAGAIN"));
 			return;
 		} else {
-			dropbear_exit("error reading");
+			dropbear_exit("error reading: %s", strerror(errno));
 		}
 	}
 
+	buf_incrpos(ses.readbuf, len);
+
 	if ((unsigned int)len == maxlen) {
 		/* The whole packet has been read */
 		decrypt_packet();
@@ -172,14 +173,14 @@ static void read_packet_init() {
 	len = read(ses.sock, buf_getwriteptr(ses.readbuf, maxlen),
 			maxlen);
 	if (len == 0) {
-		dropbear_close("remote host closed connection");
+		session_remoteclosed();
 	}
 	if (len < 0) {
 		if (errno == EINTR) {
 			TRACE(("leave read_packet_init: EINTR"));
 			return;
 		}
-		dropbear_exit("error reading");
+		dropbear_exit("error reading: %s", strerror(errno));
 	}
 
 	buf_incrwritepos(ses.readbuf, len);
@@ -315,6 +316,7 @@ static int checkmac(buffer* macbuf, buffer* sourcebuf) {
 	unsigned char macsize;
 	hmac_state hmac;
 	unsigned char tempbuf[MAX_MAC_LEN];
+	unsigned long hashsize;
 	int len;
 
 	macsize = ses.keys->recv_algo_mac->hashsize;
@@ -344,7 +346,8 @@ static int checkmac(buffer* macbuf, buffer* sourcebuf) {
 		dropbear_exit("HMAC error");
 	}
 
-	if (hmac_done(&hmac, tempbuf) != CRYPT_OK) {
+	hashsize = sizeof(tempbuf);
+	if (hmac_done(&hmac, tempbuf, &hashsize) != CRYPT_OK) {
 		dropbear_exit("HMAC error");
 	}
 
@@ -429,6 +432,14 @@ void process_packet() {
 			dropbear_close("Disconnect received");
 	}
 
+	/* Check if we should ignore this packet. Used currently only for
+	 * KEX code, with first_kex_packet_follows */
+	if (ses.ignorenext) {
+		TRACE(("Ignoring packet, type = %d", type));
+		ses.ignorenext = 0;
+		goto out;
+	}
+
 	/* check that we aren't expecting a particular packet */
 	if (ses.expecting && ses.expecting != type) {
 		/* TODO send disconnect? */
@@ -470,13 +481,14 @@ void process_packet() {
 		case SSH_MSG_CHANNEL_CLOSE:
 		case SSH_MSG_CHANNEL_OPEN_CONFIRMATION:
 		case SSH_MSG_CHANNEL_OPEN_FAILURE:
+		case SSH_MSG_GLOBAL_REQUEST:
 			/* these should be checked for authdone below */
 			process_postauth_packet(type);
 			break;
 	
 		default:
 			/* TODO this possibly should be handled */
-			TRACE(("unknown packet"));
+			TRACE(("preauth unknown packet"));
 			recv_unimplemented();
 			break;
 	}
@@ -492,6 +504,9 @@ out:
 static void process_postauth_packet(unsigned int type) {
 
 	/* messages following here require userauth before use */
+
+	/* IF YOU ADD MORE PACKET TYPES, MAKE SURE THEY'RE ALSO ADDED TO THE LIST
+	 * IN process_packet() XXX XXX XXX */
 	if (!ses.authstate.authdone) {
 		dropbear_exit("received message %d before userauth", type);
 	}
@@ -506,6 +521,13 @@ static void process_postauth_packet(unsigned int type) {
 			recv_msg_channel_window_adjust();
 			break;
 
+#ifndef DISABLE_REMOTETCPFWD
+		case SSH_MSG_GLOBAL_REQUEST:
+			/* currently only used for remote tcp, so we cheat a little */
+			recv_msg_global_request_remotetcp();
+			break;
+#endif
+
 		case SSH_MSG_CHANNEL_REQUEST:
 			recv_msg_channel_request();
 			break;
@@ -671,6 +693,7 @@ static void writemac(buffer * outputbuffer, buffer * clearwritebuf) {
 
 	int macsize;
 	unsigned char seqbuf[4];
+	unsigned long hashsize;
 	hmac_state hmac;
 
 	TRACE(("enter writemac"));
@@ -701,7 +724,8 @@ static void writemac(buffer * outputbuffer, buffer * clearwritebuf) {
 			dropbear_exit("HMAC error");
 		}
 	
-		if (hmac_done(&hmac, buf_getwriteptr(outputbuffer, macsize)) 
+		hashsize = macsize;
+		if (hmac_done(&hmac, buf_getwriteptr(outputbuffer, macsize), &hashsize) 
 				!= CRYPT_OK) {
 			dropbear_exit("HMAC error");
 		}
