@@ -52,10 +52,18 @@ struct RunOpts {
 #endif
 
 	int norootlogin;
+
+	int noauthpass;
+	int norootpass;
+
+	int nolocaltcp;
+	int noremotetcp;
 	
 	sign_key *hostkey;
 	buffer * banner;
 
+	int inetdmode;
+
 };
 
 typedef struct RunOpts runopts;
