@@ -33,6 +33,7 @@
 #include "channel.h"
 #include "queue.h"
 #include "runopts.h"
+#include "remotetcpfwd.h"
 
 extern int sessinitdone;
 extern int exitflag;
@@ -74,7 +75,6 @@ struct sshsession {
 	long connecttime; /* time of initial connection */
 
 	struct sockaddr *remoteaddr; /* the host and port of the client */
-	unsigned char *addrstring; /* the text version of remoteaddr */
 	unsigned char *hostname; /* the remote hostname */
 
 	int maxfd; /* the maximum file descriptor to check with select() */
@@ -88,6 +88,10 @@ struct sshsession {
 
 	unsigned char expecting; /* byte indicating what packet we expect next, 
 								or 0x00 for any */
+
+	unsigned char ignorenext; /* whether to ignore the next packet,
+								 used for kex_follows stuff */
+
 	
 	/* unencrypted write payload */
 	buffer *writepayload; /* this will actually refer to within clearwritebuf */
@@ -123,6 +127,11 @@ struct sshsession {
 
 	struct ChildPid * childpids; /* array of mappings childpid<->channel */
 	unsigned int childpidsize;
+	
+#ifndef DISABLE_REMOTETCPFWD
+	struct TCPListener ** tcplisteners;
+	unsigned int tcplistensize;
+#endif
 
 };
 
